namespace OutLookPlugin
{
   using System;
   using Microsoft.Office.Core;
   using Extensibility;
   using System.Runtime.InteropServices;

   #region Read me for Add-in installation and setup information.
   // When run, the Add-in wizard prepared the registry for the Add-in.
   // At a later time, if the Add-in becomes unavailable for reasons such as:
   //    1) You moved this project to a computer other than which is was originally created on.
   //    2) You chose 'Yes' when presented with a message asking if you wish to remove the Add-in.
   //    3) Registry corruption.
   // you will need to re-register the Add-in by building the <addin_name>Setup project 
   // by right clicking the project in the Solution Explorer, then choosing install.
   #endregion

   /// <summary>
   ///   The object for implementing an Add-in.
   /// </summary>
   /// <seealso class='IDTExtensibility2' />
   [GuidAttribute("78C3B5CD-3B48-4986-BEC5-D96E194DB0D7"), ProgId("OutLookPlugin.Connect")]
   public class Connect : Object, Extensibility.IDTExtensibility2
   {
      /// <summary>
      ///      Implements the constructor for the Add-in object.
      ///      Place your initialization code within this method.
      /// </summary>
      public Connect()
      {
      }

      /// <summary>
      ///      Implements the OnConnection method of the IDTExtensibility2 interface.
      ///      Receives notification that the Add-in is being loaded.
      /// </summary>
      /// <param term='application'>
      ///      Root object of the host application.
      /// </param>
      /// <param term='connectMode'>
      ///      Describes how the Add-in is being loaded.
      /// </param>
      /// <param term='addInInst'>
      ///      Object representing this Add-in.
      /// </param>
      /// <seealso class='IDTExtensibility2' />
      public void OnConnection(object application, Extensibility.ext_ConnectMode connectMode, object addInInst, ref System.Array custom)
      {
         m_objApplication = application;
         m_objAddInInstance = addInInst;
      }

      /// <summary>
      ///      Implements the OnDisconnection method of the IDTExtensibility2 interface.
      ///      Receives notification that the Add-in is being unloaded.
      /// </summary>
      /// <param term='disconnectMode'>
      ///      Describes how the Add-in is being unloaded.
      /// </param>
      /// <param term='custom'>
      ///      Array of parameters that are host application specific.
      /// </param>
      /// <seealso class='IDTExtensibility2' />
      public void OnDisconnection(Extensibility.ext_DisconnectMode disconnectMode, ref System.Array custom)
      {
      }

      /// <summary>
      ///      Implements the OnAddInsUpdate method of the IDTExtensibility2 interface.
      ///      Receives notification that the collection of Add-ins has changed.
      /// </summary>
      /// <param term='custom'>
      ///      Array of parameters that are host application specific.
      /// </param>
      /// <seealso class='IDTExtensibility2' />
      public void OnAddInsUpdate(ref System.Array custom)
      {
      }

      /// <summary>
      ///      Implements the OnStartupComplete method of the IDTExtensibility2 interface.
      ///      Receives notification that the host application has completed loading.
      /// </summary>
      /// <param term='custom'>
      ///      Array of parameters that are host application specific.
      /// </param>
      /// <see also class='IDTExtensibility2' />
      public void OnStartupComplete(ref System.Array custom)
      {
         long lSessionID = -1;
         Seapine.ttsoapcgi cgiEngine = new Seapine.ttsoapcgi();
         try
         {
            string strLoginName = "Administrator";
            try
            {
               lSessionID = cgiEngine.DatabaseLogon( "Sample Database", strLoginName, "");
            }
            catch (Exception)
            {
               lSessionID = -1;
            }

            // If logon failed then let's just get out of here.
            if (lSessionID == -1) { return; }

            // Let's find this user in TTP, and see if they actually have any defects assigned to them.
            string strDisplayName = "";
            long lNumAssigned = 0;
            GetLoggedInUserInfo( lSessionID, cgiEngine, strLoginName, ref strDisplayName, ref lNumAssigned);
            if (strDisplayName.Length == 0 || lNumAssigned == 0) { return; }

            Seapine.CTableColumn[] acolumns = null;
            PopulateSearchColumns(ref acolumns);

            Seapine.CRecordListSoap reclistDefects = cgiEngine.getRecordListForTable( lSessionID, "Defect", null, acolumns);

            foreach ( Seapine.CRecordRowSoap recrowDefect in reclistDefects.records)
            {
               try
               {
                  if (recrowDefect.row[6].value != null && recrowDefect.row[6].value.IndexOf( strDisplayName) != -1)
                     CreateOutlookTaskFromDefectRecord( lSessionID, cgiEngine, recrowDefect);
               }
               catch (Exception) {}
            }
         }
         catch (Exception)
         {
         }

         cgiEngine.DatabaseLogoff( lSessionID);
      }

      /// <summary>
      ///      Implements the OnBeginShutdown method of the IDTExtensibility2 interface.
      ///      Receives notification that the host application is being unloaded.
      /// </summary>
      /// <param term='custom'>
      ///      Array of parameters that are host application specific.
      /// </param>
      /// <seealso class='IDTExtensibility2' />
      public void OnBeginShutdown(ref System.Array custom)
      {
      }

      protected void PopulateSearchColumns(ref Seapine.CTableColumn[] p_acolumns)
      {
         long lSize = 7;
         p_acolumns = new Seapine.CTableColumn[lSize];

         long lIter = 0;
         for (; lIter < lSize; ++lIter)
            p_acolumns[lIter] = new Seapine.CTableColumn();

         // If you change the ordering of any existing elements, then you'll have to make
         // the appropriate changes elsewhere or stuff won't work as intended.
         lIter = 0;
         p_acolumns[lIter++].name = "Number";
         p_acolumns[lIter++].name = "Summary";
         p_acolumns[lIter++].name = "Description";
         p_acolumns[lIter++].name = "Steps to Reproduce";
         p_acolumns[lIter++].name = "Priority";                   // 5
         p_acolumns[lIter++].name = "Current Assignment Date";
         p_acolumns[lIter++].name = "Currently Assigned To";
      }

      protected void CreateOutlookTaskFromDefectRecord(long p_lSessionID, Seapine.ttsoapcgi p_cgiEngine, Seapine.CRecordRowSoap p_recrowDefect)
      {
         try
         {
            Outlook.Application objApplication = new Outlook.ApplicationClass();
            Outlook.NameSpace objNS  = (Outlook.NameSpace) objApplication.GetNamespace( "MAPI");
            Outlook.MAPIFolder objFolder = (Outlook.MAPIFolder) objNS.GetDefaultFolder( Outlook.OlDefaultFolders.olFolderTasks);

            // TODO: Could probably attached Defect's attachments to the new task as attachments.

            // TODO: Right now, only including the defect's first ReportedBy record in the Task body.

            // TODO: If this just searched for the <...> token, it would find a defect that already
            //       has a task, but someone had since changed the defect's Summary. Right now, if
            //       you change a defect's Summary, it's going to get a new Task in Outlook.
            string strSubject = "<Defect# " + Convert.ToInt32( p_recrowDefect.row[0].value) + ">  " + p_recrowDefect.row[1].value;
            string strSubjectToSearch = "[Subject] = \"" + strSubject + " *\"";

            Outlook.TaskItem objTask = (Outlook.TaskItem )objFolder.Items.Find( strSubjectToSearch);
            if ( objTask == null)
               objTask = (Outlook.TaskItem ) objApplication.CreateItem( Outlook.OlItemType.olTaskItem);

            objTask.Subject = strSubject;

            // TODO: Would be nice if the headers could be in bold, assuming it's possible from here.
            objTask.Body = "Description:\n\n" + (string)p_recrowDefect.row[2].value +
                           "\n\n\nSteps To Reproduce:\n\n" + (string)p_recrowDefect.row[3].value;
            objTask.StartDate = Convert.ToDateTime(p_recrowDefect.row[5].value);
            objTask.Categories = "TTPro Defect";
            objTask.Save();
         }
         catch (Exception)
         {
         }
      }

      protected bool GetLoggedInUserInfo(long p_lSessionID,
                                         Seapine.ttsoapcgi p_cgiEngine,
                                         string p_strLoginName,
                                         ref string p_strDisplayName,
                                         ref long p_lNumAssigned)
      {
         bool fResult = false;

         try
         {
            Seapine.CTableColumn[] acolumns = new Seapine.CTableColumn[4];
            acolumns[0] = new Seapine.CTableColumn();
            acolumns[0].name = "Name";
            acolumns[1] = new Seapine.CTableColumn();
            acolumns[1].name = "Assigned";
            acolumns[2] = new Seapine.CTableColumn();
            acolumns[2].name = "Active";
            acolumns[3] = new Seapine.CTableColumn();
            acolumns[3].name = "Type";

            Seapine.CRecordListSoap reclistUsers = p_cgiEngine.getRecordListForTable( p_lSessionID, "User", null, acolumns);

            Seapine.CUser user;
            foreach ( Seapine.CRecordRowSoap recrowUser in reclistUsers.records)
            {
               if (recrowUser.row[2].value == "Yes" && recrowUser.row[3].value == "Global")
               {  // User is global and active, so it could be our dude!
                  string strFName = "", strLName = "";

                  int iFindPos = recrowUser.row[0].value.IndexOf(",");

                  if (iFindPos == -1) // No last name given for TTP User
                     strFName = recrowUser.row[0].value;
                  else if (iFindPos >= 0)
                  {
                     strLName = recrowUser.row[0].value.Substring(0, iFindPos);
                     strFName = recrowUser.row[0].value.Substring(iFindPos + 2, recrowUser.row[0].value.Length - (iFindPos + 2));
                  }

                  user = p_cgiEngine.getUser( p_lSessionID, strFName, strLName);
                  if (user != null && String.Compare(user.loginname, p_strLoginName, true) == 0)
                  {
                     p_strDisplayName = recrowUser.row[0].value;
                     p_lNumAssigned = Convert.ToInt32(recrowUser.row[1].value);
                     fResult = true;
                     break;
                  }
               }
            }
         }
         catch (Exception)
         {
         }

         return (fResult);
      }

      private object m_objApplication;
      private object m_objAddInInstance;
   }
}
